@@ -2,17 +2,18 @@ package diff
 
 import (
 	"errors"
+	"fmt"
 	"io"
 	"strings"
 	"testing"
 
 	"github.com/stretchr/testify/require"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/gittest"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/diff"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/gitaly/storage"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/structerr"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/testhelper"
-	"gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/gittest"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/diff"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/gitaly/storage"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/structerr"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/testhelper"
+	"gitlab.com/gitlab-org/gitaly/v18/proto/go/gitalypb"
 )
 
 func TestCommitDiff(t *testing.T) {
@@ -548,6 +549,54 @@ func TestCommitDiff(t *testing.T) {
 				}
 			},
 		},
+		{
+			desc: "with submodule",
+			setup: func() setupData {
+				repoProto, repoPath := gittest.CreateRepository(t, ctx, cfg)
+
+				blob1 := gittest.WriteBlob(t, cfg, repoPath, []byte("foo\n"))
+				commit1 := gittest.WriteCommit(t, cfg, repoPath, gittest.WithTreeEntries(
+					gittest.TreeEntry{Path: ".gitmodules", Mode: "100644", OID: blob1},
+				))
+
+				blob2 := gittest.WriteBlob(t, cfg, repoPath, []byte("bar\n"))
+				submodule := gittest.WriteCommit(t, cfg, repoPath)
+				commit2 := gittest.WriteCommit(t, cfg, repoPath, gittest.WithTreeEntries(
+					gittest.TreeEntry{Path: ".gitmodules", Mode: "100644", OID: blob2},
+					gittest.TreeEntry{Path: "submodule", Mode: "160000", OID: submodule},
+				))
+
+				return setupData{
+					request: &gitalypb.CommitDiffRequest{
+						Repository:        repoProto,
+						LeftCommitId:      commit1.String(),
+						RightCommitId:     commit2.String(),
+						WhitespaceChanges: gitalypb.CommitDiffRequest_WHITESPACE_CHANGES_IGNORE_ALL,
+						DiffMode:          gitalypb.CommitDiffRequest_DEFAULT,
+					},
+					expectedDiff: []*diff.Diff{
+						{
+							OldMode:  0o100644,
+							NewMode:  0o100644,
+							FromID:   blob1.String(),
+							ToID:     blob2.String(),
+							FromPath: []byte(".gitmodules"),
+							ToPath:   []byte(".gitmodules"),
+							Patch:    []byte("@@ -1 +1 @@\n-foo\n+bar\n"),
+						},
+						{
+							OldMode:  0o0,
+							NewMode:  0o160000,
+							FromID:   gittest.DefaultObjectHash.ZeroOID.String(),
+							ToID:     submodule.String(),
+							FromPath: []byte("submodule"),
+							ToPath:   []byte("submodule"),
+							Patch:    fmt.Appendf(nil, "@@ -0,0 +1 @@\n+Subproject commit %s\n", submodule.String()),
+						},
+					},
+				}
+			},
+		},
 	} {
 		t.Run(tc.desc, func(t *testing.T) {
 			t.Parallel()
@@ -1134,7 +1183,7 @@ func TestCommitDiff_nonexistentCommit(t *testing.T) {
 	})
 	require.NoError(t, err)
 
-	testhelper.RequireGrpcError(t, structerr.NewFailedPrecondition("exit status 128"), drainCommitDiffResponse(stream))
+	testhelper.RequireGrpcError(t, structerr.NewFailedPrecondition("eachDiff: exit status 128"), drainCommitDiffResponse(stream))
 }
 
 func getDiffsFromCommitDiffClient(t *testing.T, client gitalypb.DiffService_CommitDiffClient) []*diff.Diff {
