@@ -8,12 +8,12 @@ import (
 	"testing"
 
 	"github.com/stretchr/testify/require"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/gitcmd"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/git/gittest"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/structerr"
-	"gitlab.com/gitlab-org/gitaly/v16/internal/testhelper"
-	"gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/gitcmd"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/git/gittest"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/structerr"
+	"gitlab.com/gitlab-org/gitaly/v18/internal/testhelper"
+	"gitlab.com/gitlab-org/gitaly/v18/proto/go/gitalypb"
 )
 
 func TestDiffBlobs(t *testing.T) {
@@ -74,6 +74,102 @@ func TestDiffBlobs(t *testing.T) {
 				}
 			},
 		},
+		{
+			desc: "rawInfo entry missing old blob ID",
+			setup: func() setupData {
+				repoProto, repoPath := gittest.CreateRepository(t, ctx, cfg)
+
+				blobID := gittest.WriteBlob(t, cfg, repoPath, []byte("bar\n"))
+
+				return setupData{
+					request: &gitalypb.DiffBlobsRequest{
+						Repository: repoProto,
+						RawInfo: []*gitalypb.ChangedPaths{
+							{
+								Status:    gitalypb.ChangedPaths_RENAMED,
+								OldBlobId: "",
+								NewBlobId: blobID.String(),
+								Path:      []byte("foo"),
+							},
+						},
+					},
+					expectedErr: structerr.NewInvalidArgument("raw info entry missing blob IDs"),
+				}
+			},
+		},
+		{
+			desc: "rawInfo entry missing new blob ID",
+			setup: func() setupData {
+				repoProto, repoPath := gittest.CreateRepository(t, ctx, cfg)
+
+				blobID := gittest.WriteBlob(t, cfg, repoPath, []byte("bar\n"))
+
+				return setupData{
+					request: &gitalypb.DiffBlobsRequest{
+						Repository: repoProto,
+						RawInfo: []*gitalypb.ChangedPaths{
+							{
+								Status:    gitalypb.ChangedPaths_RENAMED,
+								OldBlobId: blobID.String(),
+								NewBlobId: "",
+								Path:      []byte("foo"),
+							},
+						},
+					},
+					expectedErr: structerr.NewInvalidArgument("raw info entry missing blob IDs"),
+				}
+			},
+		},
+		{
+			desc: "rawInfo rename entry missing path",
+			setup: func() setupData {
+				repoProto, repoPath := gittest.CreateRepository(t, ctx, cfg)
+
+				blobID1 := gittest.WriteBlob(t, cfg, repoPath, []byte("foo\n"))
+				blobID2 := gittest.WriteBlob(t, cfg, repoPath, []byte("bar\n"))
+
+				return setupData{
+					request: &gitalypb.DiffBlobsRequest{
+						Repository: repoProto,
+						RawInfo: []*gitalypb.ChangedPaths{
+							{
+								Status:    gitalypb.ChangedPaths_RENAMED,
+								OldBlobId: blobID1.String(),
+								NewBlobId: blobID2.String(),
+								Path:      nil,
+								OldPath:   []byte("bar"),
+							},
+						},
+					},
+					expectedErr: structerr.NewInvalidArgument("raw info entry missing path"),
+				}
+			},
+		},
+		{
+			desc: "rawInfo rename entry missing old path",
+			setup: func() setupData {
+				repoProto, repoPath := gittest.CreateRepository(t, ctx, cfg)
+
+				blobID1 := gittest.WriteBlob(t, cfg, repoPath, []byte("foo\n"))
+				blobID2 := gittest.WriteBlob(t, cfg, repoPath, []byte("bar\n"))
+
+				return setupData{
+					request: &gitalypb.DiffBlobsRequest{
+						Repository: repoProto,
+						RawInfo: []*gitalypb.ChangedPaths{
+							{
+								Status:    gitalypb.ChangedPaths_RENAMED,
+								OldBlobId: blobID1.String(),
+								NewBlobId: blobID2.String(),
+								Path:      []byte("foo"),
+								OldPath:   nil,
+							},
+						},
+					},
+					expectedErr: structerr.NewInvalidArgument("rename/copy raw info entry missing old path"),
+				}
+			},
+		},
 		{
 			desc: "commit ID in request",
 			setup: func() setupData {
@@ -438,20 +534,9 @@ func TestDiffBlobs(t *testing.T) {
 							},
 						},
 					},
-					expectedResponses: []*gitalypb.DiffBlobsResponse{
-						{
-							LeftBlobId:  blobID1.String(),
-							RightBlobId: blobID2.String(),
-							Status:      gitalypb.DiffBlobsResponse_STATUS_END_OF_PATCH,
-						},
-						{
-							LeftBlobId:  blobID1.String(),
-							RightBlobId: blobID3.String(),
-							Patch:       []byte("@@ -1 +1 @@\n-foo\n+ foo \n"),
-							PatchSize:   24,
-							Status:      gitalypb.DiffBlobsResponse_STATUS_END_OF_PATCH,
-						},
-					},
+					expectedErr: testhelper.ToInterceptedMetadata(
+						structerr.NewInvalidArgument("whitespace changes cannot be ignored when blob pairs are provided"),
+					),
 				}
 			},
 		},
@@ -479,18 +564,9 @@ func TestDiffBlobs(t *testing.T) {
 							},
 						},
 					},
-					expectedResponses: []*gitalypb.DiffBlobsResponse{
-						{
-							LeftBlobId:  blobID1.String(),
-							RightBlobId: blobID2.String(),
-							Status:      gitalypb.DiffBlobsResponse_STATUS_END_OF_PATCH,
-						},
-						{
-							LeftBlobId:  blobID1.String(),
-							RightBlobId: blobID3.String(),
-							Status:      gitalypb.DiffBlobsResponse_STATUS_END_OF_PATCH,
-						},
-					},
+					expectedErr: testhelper.ToInterceptedMetadata(
+						structerr.NewInvalidArgument("whitespace changes cannot be ignored when blob pairs are provided"),
+					),
 				}
 			},
 		},
@@ -1130,19 +1206,20 @@ func TestDiffBlobs(t *testing.T) {
 
 			var actualResp []*gitalypb.DiffBlobsResponse
 			for {
-				resp, err := stream.Recv()
-				if errors.Is(err, io.EOF) {
-					break
-				}
+				var resp *gitalypb.DiffBlobsResponse
 
-				testhelper.RequireGrpcError(t, data.expectedErr, err)
+				resp, err = stream.Recv()
 				if err != nil {
+					if errors.Is(err, io.EOF) {
+						err = nil
+					}
 					break
 				}
 
 				actualResp = append(actualResp, resp)
 			}
 
+			testhelper.RequireGrpcError(t, data.expectedErr, err)
 			testhelper.ProtoEqual(t, data.expectedResponses, actualResp)
 		})
 	}
